/*
    Simple DPlot plugin for the Generate menu. The compiled dll should be placed
	in <dplot folder>/plugins/generate.

    Only 2 functions are required for "Generate" plugins:
	plugin_init provides:
		1) A string description of the plugin that will appear in the Generate menu.
		2) An "action" code that describes how plugin_run operates on the input
		   data. Value should be one of (defined in dplot.h):
		   PLUGIN_REPLACEDATA  Plugin operates on and replaces existing data in the
		                       same document.
	       PLUGIN_NEWDATA      Plugin operates on existing data and returns one or more 
		                       data sets (curves) for the same document.
		   PLUGIN_NEWDOCUMENT  Plugin operates on existing data (or not) and returns 
		                       an entirely new document. In this case the input provided
							   by DPlot is a *copy* of an existing document - plugin
							   does not have to concern itself with opening a new window.
		All of the above should be comined with PLUGIN_DOUBLES for double-
		precision (16 byte) data.

		plugin_init should return 1 for success, 0 for failure (though there's currently
		no way for it to fail).

	plugin_run modifies existing data or generates new data.
		For "actions" that create new data values, plugin_run *must* check to insure that 
		the input X and Y arrays are large enough to accomodate the new data. If they are 
		not, the plugin should fail with a return code of PLUGIN_BADARRAYSIZE and return
		the required size in the MaxCurves and MaxPoints members of the DPLOT structure.

		If plugin_run opens a dialog box or other window (including MessageBox), it should 
		first disable DPlot as shown below. Otherwise the user will be able to run multiple 
		instances of the plugin or perform other unintended operations. If DPlot is disabled,
		though, make *sure* the plugin re-enables it before returning or you'll have an
		unhappy user.
*/
#include <windows.h>
#include "dplot.h"

__declspec(dllexport) int plugin_init( char *Desc, char *Unused, int *Action )

{
	lstrcpy(Desc,"Invert Amplitudes");		// Description appearing in DPlot menu
	*Action = PLUGIN_REPLACEDATA;			// See DPLOT.H
	return 1;
}
//==============================================================================
__declspec(dllexport) int plugin_version()

{
	return (int)DPLOT_VERSION;
}
//==============================================================================
__declspec(dllexport) int plugin_run( HWND hwndOwner, DPLOT *DPlot, float *X, float *Y)
{
	int RetCode = PLUGIN_FAILURE;
	int i;
	int j;
	int j1;

	if(DPlot->Version != DPLOT_VERSION)
	{
		MessageBox(hwndOwner,"This plugin requires DPlot 1.8 or later.","DPlot Plugin",MB_ICONEXCLAMATION);
		goto PluginExit;
	}
	EnableWindow(hwndOwner,FALSE);

	if(DPlot->NumCurves <= 0) goto PluginExit;

	// DPLOT *should* insure that this plugin is only called with
	// XY plots, but it never hurts to be safe:
	if(DPlot->DataFormat == DATA_3D)  goto PluginExit;
	if(DPlot->DataFormat == DATA_3DR) goto PluginExit;
	if(!Y) goto PluginExit;

	// This plugin just inverts all amplitudes
	j1 = 0;
	for(i=0; i< (int)DPlot->NumCurves; i++) {
		for(j=j1; j<j1+(int)DPlot->NP[i]; j++) {
			Y[j] = -Y[j];
		}
		j1 += DPlot->MaxPoints;
	}	 

	lstrcpy(DPlot->Title[2],"Data modified by INVERT.DLL");

	RetCode = PLUGIN_SUCCESS;

PluginExit:
	// Re-enable owner
	EnableWindow(hwndOwner,TRUE);
	return RetCode;
}

